﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Docking;

namespace RadDockingWPF_1057940
{
    //Please, notice that this is a custom implementation and you might need to improve it in order to achieve the desired behavior of the unpinned Panes!
    public class CustomIFlyoutBehavior : IFlyoutBehavior
    {
        private RadPane nextPaneToActivate;
        private DispatcherTimer timer;
        private IFlyoutHost currentHost;

        public CustomIFlyoutBehavior()
        {
            this.timer = new DispatcherTimer()
            {
                Interval = TimeSpan.FromMilliseconds(800)
            };

            this.timer.Tick += timer_Tick;
        }

        //You could add attitional timer that will increase the time of showing the Panes
        void timer_Tick(object sender, EventArgs e)
        {
            currentHost.StartOpenTimer();
            timer.Stop();
        }

        void IFlyoutBehavior.OnMouseEnter(IFlyoutHost host, RadPane targetPane)
        {
            host.SetSelectedPane(targetPane);

            if (host.CurrentState == FlyoutState.Closed)
            {
                this.currentHost = host;
                this.timer.Start();
            }
            else if (host.CurrentState == FlyoutState.ClosingTimer)
            {
                host.CancelCloseTimer();
            }
        }

        void IFlyoutBehavior.OnMouseLeave(IFlyoutHost host)
        {
            timer.Stop();
            var selectedPane = host.SelectedPane;

            // You should close the pane even if it active in order to close it as expected when the Pane is focused
            //if (selectedPane != null && !selectedPane.IsActive)
            if (selectedPane != null)
            {
                if (host.CurrentState == FlyoutState.Opened || host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    host.StartCloseTimer();
                    
                    selectedPane.IsActive = false;
                }
                else if (host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    host.StartCloseAnimation();
                }
                else if (host.CurrentState == FlyoutState.OpeningTimer)
                {
                    host.CancelOpenTimer();
                    host.SetSelectedPane(null);
                }
            }
        }

        void IFlyoutBehavior.OnOpeningTimerTimeout(IFlyoutHost host)
        {
            timer.Stop();
            host.StartOpenAnimation();
        }

        void IFlyoutBehavior.OnClosingTimerTimeout(IFlyoutHost host)
        {
            host.StartCloseAnimation();
        }

        void IFlyoutBehavior.OnPaneActivated(IFlyoutHost host, RadPane targetPane)
        {
            host.SetSelectedPane(targetPane);

            if (host.CurrentState == FlyoutState.Closed)
            {
                host.StartOpenAnimation();
            }
            else if (host.CurrentState == FlyoutState.ClosingTimer)
            {
                host.CancelCloseTimer();
            }
        }

        void IFlyoutBehavior.OnPaneDeactivated(IFlyoutHost host, RadPane targetPane)
        {
            var selectedPane = host.SelectedPane;

            //if (selectedPane != null && !selectedPane.IsActive)
            if (selectedPane != null)
            {
                if (host.CurrentState == FlyoutState.Opened || host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    if (this.nextPaneToActivate == null || (targetPane != null && this.nextPaneToActivate.Parent != targetPane.Parent))
                    {
                        host.StartCloseAnimation();
                    }
                }
                else if (host.CurrentState == FlyoutState.OpeningTimer)
                {
                    host.CancelOpenTimer();
                    host.SetSelectedPane(null);
                }
            }
        }

        void IFlyoutBehavior.OnPaneMouseLeftButtonDown(IFlyoutHost host, RadPane targetPane)
        {
            try
            {
                this.timer.Stop();
                this.nextPaneToActivate = targetPane;
                host.ActivatePane(targetPane);
            }
            finally
            {
                this.nextPaneToActivate = null;
            }
        }
    }
}
